﻿if not ClassTimer then return end
local BS = AceLibrary("Babble-Spell-2.2")
local L = AceLibrary("AceLocale-2.2"):new("ClassTimer")
local candy = AceLibrary("CandyBar-2.0")

ClassTimer.Rules = {}
ClassTimer.RuleMenu = {}
ClassTimer.timers = {}

function ClassTimer:RulesInit()
	self:CreateTimers()
	self.timers = nil
	for k in pairs(self.RuleMenu) do
		self.Menu.args[k] = {
			type = "group",
			order = 4,
			name = k,
			desc = L["Which Buffs to show."],
			args = {}
		}
		for v in pairs(self.RuleMenu[k]) do
			if self.Rules[v].Alt then name = BS[self.Rules[v].Alt] else name = v end
				self.Menu.args[k].args[name] = {
					order = 3,
					type = 'toggle',
					name = name,
				    desc = L["Enable/Dissable the timer"],
				    get = function()
				        return self.db.class[v]
				    end,
				    set = function(u)
						self.db.class[v] = u
					end,
				}
		end
	end
	self.RuleMenu = nil	
end

function ClassTimer:RulesSetup()
	if self.OnUpdate then
		self:ScheduleRepeatingEvent(ClassTimer.OnUpdate, 0.5, self)
	end
end

function ClassTimer:AddRule(group, name, table)
	if not table then table = {} end
	if name ~= "PvP" and name ~= "PvE" then
		name = BS[name]
	end
	local group = L[group]
	self.Rules[name] = table
	if table.NoVar then return end
	if not self.RuleMenu[group] then self.RuleMenu[group] = {} end
	self.RuleMenu[group][name] = table
	self.defaults.rules[name] = true
end

local math_floor = math.floor
function ClassTimer:SetupBuffs(UnitID)
	local name, rank, icon, count, debuffType, duration, timeLeft, time, oldtime
	local noupdate = (UnitID ~= "player" and UnitIsUnit(UnitID, "player")) or ( UnitID ~= "player" and UnitID ~= "target" and UnitIsUnit(UnitID, "target")) or (UnitInRaid(UnitID) and string.find(UnitID, "party"))
	for i = 1, 32 do
		name, rank, icon, count, duration, timeLeft = UnitBuff(UnitID, i)
		if not timeLeft or timeLeft == 0 or noupdate then 
			local n = i
			while self[UnitID].buffs[n] do
				self[UnitID].buffs[n] = nil 
				self:DebuffFade(UnitID, n)
				n = n +1
			end
			break
		end
		time = math_floor(GetTime() + timeLeft)
		oldtime = math_floor(self[UnitID].buffs[i] or 0)
		if time ~= oldtime then
			self[UnitID].buffs[i] = time
			self:BuffUpdate(UnitID, i, name, icon, duration, timeLeft)
		end
	end
end

function ClassTimer:SetupDebuffs(UnitID)
	local name, rank, icon, count, debuffType, duration, timeLeft, time, oldtime
	local noupdate = (UnitID ~= "player" and UnitIsUnit(UnitID, "player")) or ( UnitID ~= "target" and UnitIsUnit(UnitID, "target"))
	for i = 1, 40 do
		name, rank, icon, count, debuffType, duration, timeLeft = UnitDebuff(UnitID, i)
		if not timeLeft or noupdate then 
			local n = i
			while self[UnitID].debuffs[n] do
				self[UnitID].debuffs[n] = nil 
				self:DebuffFade(UnitID, n)
				n = n +1
			end
			break
		end
		time = math_floor(GetTime() + timeLeft)
		oldtime = math_floor(self[UnitID].debuffs[i] or 0)
		if time ~= oldtime then
			self[UnitID].debuffs[i] = time
			self:DebuffUpdate(UnitID, i, name, icon, count, duration, timeLeft)
		end
	end

end

function ClassTimer:DebuffFade(UnitID, id)
	self:UnregisterCandyBar("ClassTimer_"..UnitID.."_debuff".. id)
end

function ClassTimer:DebuffUpdate(UnitID, id, name, icon, count, duration, timeLeft)
	local id = "ClassTimer_"..UnitID.."_debuff"..id
	if self.Rules[name] and self.Rules[name].Alt then name = self.Rules[name].Alt end
	if not candy:IsCandyBarRegistered(id) and self.db.class[name] then
		self:Bars(id, name, UnitName(UnitID), count, icon, duration, timeLeft)
	elseif self.db.class[name] then
		candy:SetCandyBarTimeLeft(id, timeLeft)
		candy:SetCandyBarText(id, self:GetText(name, UnitName(UnitID), count))
	end
end

function ClassTimer:BuffFade(UnitID, id)
	self:UnregisterCandyBar("ClassTimer_"..UnitID.."_buff"..id)
end

function ClassTimer:BuffUpdate(UnitID, id, name, icon, duration, timeLeft)
	local id = "ClassTimer_"..UnitID.."_buff"..id
	if self.Rules[name] and self.Rules[name].Alt then name = self.Rules[name].Alt end
	if not candy:IsCandyBarRegistered(name) and self.db.class[name] then
		self:Bars(id, name, UnitID ~= "player" and UnitName(UnitID), nil, icon, duration, timeLeft)
	elseif self.db.class[name] then
		candy:SetCandyBarTimeLeft(id, timeLeft)
	end
end

function ClassTimer:Bars(id, move, target, applications, path, time, timeleft)
if self.Rules[move] and self.Rules[move].Alt then move = self.Rules[move].Alt end
ClassTimer:StartBar(id, move, target, applications, time, timeleft, path)
end 